split_helper = {}

function split_helper.split(String, Sep, MaxSplit)
   -- for empty string, returns zero
   -- generally, returns number of separator occurrences + 1 even if head or tail are empty

   -- separator must be non-empty for this logic to work;
   -- otherwise it would be something else we don't care to define
   assert (Sep:len() >= 1 )

   if not String:len() then
      -- splitting an empty string provides zero chunks
      return {}
   end
   
   local Next = 1 -- scanning index - pos of last separator + 1
   local ChunkList = {}
   while (not MaxSplit or #ChunkList < MaxSplit) do
      local SepStart, SepEnd = String:find(Sep, Next, true)
      if not SepStart then
         -- no more separator -
         -- take what we have left as the last chunk
         break
      end
      table.insert(ChunkList, String:sub(Next, SepStart-1))
      Next = SepEnd+1
   end
   table.insert(ChunkList, String:sub(Next))
   return ChunkList
end

local split = split_helper.split

local function stringListEqual(List1, List2)
   if #List1 ~= #List2 then
      return false
   end
   local Index
   for Index = 1,#List1 do
      if List1[Index] ~= List2[Index] then
         return false
      end
   end
   return true
end

function split_helper.unittest()
   local same = stringListEqual
   assert( same( split('a|b|c', '|'), {'a','b','c'} ) )
   assert( same( split('|b|', '|'), {'','b',''} ) )
   assert( same( split('a|b|c', '{}'), {'a|b|c'} ) )      
   assert( same( split('a{}b{}c', '{}'), {'a','b','c'} ) )   
   assert( same( split('||', '|'), {'','',''} ) )
   assert( same( split('a=b=c=d', '=',0), {'a=b=c=d'} ) )
   assert( same( split('a=b=c=d', '=',1), {'a', 'b=c=d'} ) )
   assert( same( split('a=b=c=d', '=',2), {'a', 'b', 'c=d'} ) )   
end

function split_helper.splitlines(String)
   -- making a simplifying assumption that newlines contain \n
   -- and let the caller deal with any discrepancies
   return split(String, '\n')
end
